@@ -36,18 +36,19 @@
 #include <algorithm>
 #include <atomic>
 #include <cstdint>
+#include <cstring>
 #include <string>
 #include <type_traits>
 #include <utility>
+#include <variant>
 
-#include "absl/status/status.h"
-#include "absl/strings/str_cat.h"
-#include "absl/strings/string_view.h"
 #include "src/core/call/message.h"
 #include "src/core/call/metadata.h"
 #include "src/core/call/metadata_batch.h"
+#include "src/core/channelz/property_list.h"
 #include "src/core/lib/promise/activity.h"
 #include "src/core/lib/promise/cancel_callback.h"
+#include "src/core/lib/promise/detail/promise_like.h"
 #include "src/core/lib/promise/map.h"
 #include "src/core/lib/promise/poll.h"
 #include "src/core/lib/promise/seq.h"
@@ -55,6 +56,12 @@
 #include "src/core/lib/surface/completion_queue.h"
 #include "src/core/util/crash.h"
 #include "src/core/util/grpc_check.h"
+#include "src/core/util/upb_utils.h"
+#include "src/proto/grpc/channelz/v2/promise.upb.h"
+#include "upb/mem/arena.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 
 namespace grpc_core {
 
@@ -238,6 +245,32 @@ class OpHandlerImpl {
     PromiseFactory promise_factory_;
     Promise promise_;
   };
+
+ public:
+  void ToProto(grpc_channelz_v2_Promise* promise_proto,
+               upb_Arena* arena) const {
+    auto* custom_promise =
+        grpc_channelz_v2_Promise_mutable_custom_promise(promise_proto, arena);
+    std::string type = absl::StrCat("OpHandlerImpl<", OpName(), ">");
+    grpc_channelz_v2_Promise_Custom_set_type(
+        custom_promise, CopyStdStringToUpbString(type, arena));
+    channelz::PropertyList properties;
+    switch (state_) {
+      case State::kDismissed:
+        properties.Set("state", "dismissed");
+        break;
+      case State::kPromiseFactory:
+        properties.Set("state", "promise_factory");
+        break;
+      case State::kPromise:
+        properties.Set("state", "promise");
+        properties.Set("promise", PromiseProperty(&promise_));
+        break;
+    }
+    properties.FillUpbProto(grpc_channelz_v2_Promise_Custom_mutable_properties(
+                                custom_promise, arena),
+                            arena);
+  }
 };
 
 template <grpc_op_type op_type, typename PromiseFactory>
@@ -289,6 +322,21 @@ class BatchOpIndex {
 // to Empty{}
 class WaitForCqEndOp {
  public:
+  struct NotStarted {
+    bool is_closure;
+    void* tag;
+    grpc_error_handle error;
+    grpc_completion_queue* cq;
+  };
+  struct Started {
+    explicit Started(Waker waker) : waker(std::move(waker)) {}
+    Waker waker;
+    grpc_cq_completion completion;
+    std::atomic<bool> done{false};
+  };
+  struct Invalid {};
+  using State = std::variant<NotStarted, Started, Invalid>;
+
   WaitForCqEndOp(bool is_closure, void* tag, grpc_error_handle error,
                  grpc_completion_queue* cq)
       : state_{NotStarted{is_closure, tag, std::move(error), cq}} {}
@@ -308,24 +356,35 @@ class WaitForCqEndOp {
   }
 
  private:
-  struct NotStarted {
-    bool is_closure;
-    void* tag;
-    grpc_error_handle error;
-    grpc_completion_queue* cq;
-  };
-  struct Started {
-    explicit Started(Waker waker) : waker(std::move(waker)) {}
-    Waker waker;
-    grpc_cq_completion completion;
-    std::atomic<bool> done{false};
-  };
-  struct Invalid {};
-  using State = std::variant<NotStarted, Started, Invalid>;
-
   static std::string StateString(const State& state);
 
   State state_{Invalid{}};
+
+ public:
+  void ToProto(grpc_channelz_v2_Promise* promise_proto,
+               upb_Arena* arena) const {
+    auto* custom_promise =
+        grpc_channelz_v2_Promise_mutable_custom_promise(promise_proto, arena);
+    grpc_channelz_v2_Promise_Custom_set_type(
+        custom_promise, CopyStdStringToUpbString("WaitForCqEndOp", arena));
+    channelz::PropertyList properties;
+    std::visit(
+        [&properties](const auto& s) {
+          using T = std::decay_t<decltype(s)>;
+          if constexpr (std::is_same_v<T, NotStarted>) {
+            properties.Set("state", "not_started");
+          } else if constexpr (std::is_same_v<T, Started>) {
+            properties.Set("state", "started");
+            properties.Set("done", s.done.load(std::memory_order_relaxed));
+          } else if constexpr (std::is_same_v<T, Invalid>) {
+            properties.Set("state", "invalid");
+          }
+        },
+        state_);
+    properties.FillUpbProto(grpc_channelz_v2_Promise_Custom_mutable_properties(
+                                custom_promise, arena),
+                            arena);
+  }
 };
 
 template <typename FalliblePart, typename FinalPart>
@@ -406,6 +465,12 @@ class PollBatchLogger {
 
   void* tag_;
   F f_;
+
+ public:
+  void ToProto(grpc_channelz_v2_Promise* promise_proto,
+               upb_Arena* arena) const {
+    PromiseAsProto(f_, promise_proto, arena);
+  }
 };
 
 template <typename F>
